import java.nio.file.*;
import java.io.IOException;
import java.util.*;
import org.chocosolver.solver.Model;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.search.limits.FailCounter;

public class ProductionHoraire {
    public static final int N_PERIODE = 16;
    public static void main(String[] args) {

	String instancePath = args[0];
	
	// lecture des lignes du fichier d'instance
	String[] variablesInstance = null;
	String[] strLigne2 = null;
	String[] strLigne3 = null;    
	try {
	    List<String> lignesInstance = Files.readAllLines(Paths.get(instancePath));
	    variablesInstance = lignesInstance.get(0).split("\\s+");
	    strLigne2 = lignesInstance.get(1).split("\\s+");
	    strLigne3 = lignesInstance.get(2).split("\\s+");
	}
	catch (IOException e){
	    System.err.println("Caught IOException: " + e.getMessage());
	}
	catch (Exception e){
	    System.err.println("Caught Exception: " + e.getMessage());
	}
	
	// traitement de la 1re ligne
	Integer N = Integer.valueOf(variablesInstance[0]);
	Integer MIN_H = Integer.valueOf(variablesInstance[1]);
	Integer MAX_H = Integer.valueOf(variablesInstance[2]);
	Integer MIN_PERIODE = Integer.valueOf(variablesInstance[3]);
	
	// traitement de la 2e ligne
	List<Integer> nbEmployesRequis = new ArrayList<>();
	for (String s: strLigne2) {
	    Integer i = Integer.valueOf(s);
	    nbEmployesRequis.add(i);
	}
	// traitement de la 3e ligne
	List<Integer> nbEmployesSouhaite = new ArrayList<>();
	for (String s: strLigne3) {
	    Integer i = Integer.valueOf(s);
	    nbEmployesSouhaite.add(i);
	}
	
	// Vérification du fichier d'instance
	System.out.println("### Debut Validation Lecture Instance ###");
	System.out.println("N="+N.toString());
	System.out.println("MIN_H="+MIN_H.toString());
	System.out.println("MAX_H="+MAX_H.toString());
	System.out.println("MIN_PERIODE="+MIN_PERIODE.toString());
	System.out.println("nbEmployesRequis="+nbEmployesRequis.toString());
	System.out.println("nbEmployesSouhaite="+nbEmployesSouhaite.toString());
	System.out.println("### Fin Validation Lecture Instance ###");

	Model model = new Model("Production Horaire");

	// Création d'une matrice de dimensions N x N_PERIODE de variables E(i,j) dont les domaines sont des variables binaires.
	// E(i,j)=0 signifie que l'employé i est au repos à la période j.
	// E(i,j)=1 signifie que l'employé i travaille à la période j.

	BoolVar[][] LignesE = model.boolVarMatrix("E",N,N_PERIODE);

	BoolVar[][] ColonnesE = new BoolVar[N_PERIODE][N];
        for (int i = 0; i < N_PERIODE; i++) {
            for (int j = 0; j < N; j++) {
                ColonnesE[i][j] = LignesE[j][i];
            }
        }

	// Contrainte du nombre requis d'employés

	IntVar[] IVnbEmployesRequis  = new IntVar[N_PERIODE];
	for (int i=0; i < N_PERIODE; i++) {
	    IVnbEmployesRequis[i] = model.intVar(nbEmployesRequis.get(i));
	    model.sum(ColonnesE[i], ">=", IVnbEmployesRequis[i]).post();
	}

	// Contrainte du motif d'horaire

	 // Creation du solveur
        Solver solver = model.getSolver();
	
	// Résolution du modèle
	solver.findSolution();
	for (int i = 0; i < N; i++) {
            for (int j = 0; j < N_PERIODE; j++) {
                System.out.print(LignesE[i][j].getValue());
                System.out.print("  ");
            }
            System.out.println("");
        }
        solver.printStatistics();
    }
}
